
subroutine CalcEnergy(Pos, AA, M, L, rc, rlj, rw, sigma, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: AA
    real(8), intent(in) :: L, rc, rlj, rw, sigma
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 236., r0 = 3.8
    !======== YOUR CODE HERE ========
    !define variables to be used in the function but not inputs or outputs
    real(8), Dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12
    real(8) :: Shiftw, Shift 
    real(8) :: rc2, rw2, rlj2
    real(8) :: sig3, sig6, sig12
    integer :: i, j
    PEnergy = 0.
    Shift = -4.*5*(2.5**(-12)-2.5**(-6))
    Shiftw = -4.*(2**(-2)-0.5)
    rc2 = rc * rc
    rlj2 = rlj*rlj
    rw2= rw*rw
        !avoid calculating sqrt to increase speed 
    sig3=sigma*sigma*sigma
    sig6=sig3*sig3
    sig12=sig6*sig6

    do i=0, NAtom-1
        Posi = Pos(i,:)
            !create temporary array to store Pos(i,:) for faster access in loop
        do j = i+1, Natom - 1
            rij = Pos(j,:) - Posi
        	rij = rij - L * dnint(rij/L)
        	d2 = sum(rij * rij)
		if (j == i+1 .and. mod(j,M)>0) then
                !this is a bonded interaction;
                !use the harmonic bond potential
                PEnergy = PEnergy +k*0.5*(sqrt(d2)-r0)*(sqrt(d2)-r0)
            else
                !this is a nonbonded interaction; 
		    !need to determine particle type to calc potential
                id2 = 1./d2
                id6 = id2 * id2 * id2     !inverse distance to the sixth
                id12 = id6 * id6          !inverse twelfth distance               
		    if ((AA(i)==1 .and. AA(j)==1) .or. (AA(i)/=AA(j))) then
			!this is modeled as a WCA potential
			if (d2 < rw2) then
				PEnergy = PEnergy + 4. * (sig12*id12 - sig6*id6) + Shiftw
				if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) /= 7)) then
					!this is screened coulomb with same charges 
					PEnergy = PEnergy + 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
				endif
				if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) == 7)) then
					!this is screened coulomb with opposite charges
					PEnergy = PEnergy - 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
				endif					
			endif
		    endif
		    if (AA(i)==2 .and. AA(j)==2) then
			!this is hydrophobic modeled as deep well LJ particle
			if (d2<rlj2) then
				PEnergy =  PEnergy + 4. * 5 * (sig12*id12 - sig6*id6) + Shift
			endif
		    endif
		    if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) /= 7)) then
			!this is screened coulomb with same charges 
			if (d2<rc2) then
				PEnergy = PEnergy + 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
			endif
		    endif
		    if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) == 7)) then
			!this is screened coulomb with opposite charges
			if (d2<rc2) then
				PEnergy = PEnergy- 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
			endif
		    endif			

		endif
        enddo
    enddo

end subroutine


subroutine CalcEnergyForces(Pos, AA, M, L, rc, rlj, rw, sigma, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: AA
    real(8), intent(in) :: L, rc, rlj, rw, sigma
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 2360., r0 = 3.8
    !======== YOUR CODE HERE ========
    !define variables to be used in the function but not inputs or outputs
    real(8), Dimension(Dim) :: rij, Posi, Fij
    real(8) :: d2, id2, id6, id12
    real(8) :: Shiftw, Shift 
    real(8) :: rc2, rw2, rlj2
    real(8) :: sig3, sig6, sig12
    integer :: i, j
    PEnergy = 0.
    Forces = 0.
    Shift = -4.*5*(2.5**(-12)-2.5**(-6))
    Shiftw = -4.*(2**(-2)-0.5)
    rc2 = rc * rc
    rlj2 = rlj*rlj
    rw2= rw*rw
        !avoid calculating sqrt to increase speed        
    sig3=sigma*sigma*sigma
    sig6=sig3*sig3
    sig12=sig6*sig6
    do i=0, NAtom-1
        Posi = Pos(i,:)
            !create temporary array to store Pos(i,:) for faster access in loop
        do j = i+1, Natom - 1
            rij = Pos(j,:) - Posi
        	rij = rij - L * dnint(rij/L)
        	d2 = sum(rij * rij)
		if (j == i+1 .and. mod(j,M)>0) then
                !this is a bonded interaction;
                !use the harmonic bond potential
                PEnergy = PEnergy +k*0.5*(sqrt(d2)-r0)*(sqrt(d2)-r0)
                Fij = k*(sqrt(d2)-r0)*rij/sqrt(d2)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            else
                !this is a nonbonded interaction; 
		    !need to determine particle type to calc potential
                id2 = 1./d2
                id6 = id2 * id2 * id2     !inverse distance to the sixth
                id12 = id6 * id6          !inverse twelfth distance               
		    if ((AA(i)==1 .and. AA(j)==1) .or. (AA(i)/=AA(j))) then
			!this is modeled as a WCA potential
			if (d2 < rw2) then
				PEnergy = PEnergy + 4. * (sig12*id12 - sig6*id6) + Shiftw
				Fij = rij *((-(48.*sig12) * id12 + (24.*sig6) * id6) * id2)
               		Forces(i,:) = Forces(i,:) + Fij
                		Forces(j,:) = Forces(j,:) - Fij
				if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) /= 7)) then
					!this is screened coulomb with same charges 
					PEnergy = PEnergy + 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
					Fij = Fij-rij*sqrt(id2)*(7.0*exp(-sqrt(d2)/8.12)*(8.12 + sqrt(d2)))/(8.12*d2)
                			Forces(i,:) = Forces(i,:) + Fij
                			Forces(j,:) = Forces(j,:) - Fij
				endif
				if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) == 7)) then
					!this is screened coulomb with opposite charges
					PEnergy = PEnergy - 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
					Fij = Fij+rij*sqrt(id2)*(7.0*exp(-sqrt(d2)/8.12)*(8.12 + sqrt(d2)))/(8.12*d2)
                			Forces(i,:) = Forces(i,:) + Fij
                			Forces(j,:) = Forces(j,:) - Fij
				endif					
			endif
		    endif
		    if (AA(i)==2 .and. AA(j)==2) then
			!this is hydrophobic modeled as deep well LJ particle
			if (d2<rlj2) then
				PEnergy =  PEnergy + 4. * 5 * (rlj*id12 - rlj*id6) + Shift
				Fij = rij *5*((-(48.*sig12)* id12 + (24.*sig6) * id6) * id2)
               		Forces(i,:) = Forces(i,:) + Fij
                		Forces(j,:) = Forces(j,:) - Fij
			endif
		    endif
		    if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) /= 7)) then
			!this is screened coulomb with same charges 
			if (d2<rc2) then
				PEnergy = PEnergy + 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
				Fij =-rij*sqrt(id2)*(7.0*exp(-sqrt(d2)/8.12)*(8.12 + sqrt(d2)))/(8.12*d2)
                		Forces(i,:) = Forces(i,:) + Fij
                		Forces(j,:) = Forces(j,:) - Fij
			endif
		    endif
		    if (AA(i) > 2 .and. AA(j) > 2 .and. (AA(i)+AA(j) == 7)) then
			!this is screened coulomb with opposite charges
			if (d2<rc2) then
				PEnergy = PEnergy- 7.0/sqrt(d2)*exp(-sqrt(d2)/8.12)
				Fij = rij*sqrt(id2)*(7.0*exp(-sqrt(d2)/8.12)*(8.12 + sqrt(d2)))/(8.12*d2)
                		Forces(i,:) = Forces(i,:) + Fij
                		Forces(j,:) = Forces(j,:) - Fij
			endif
		    endif					

		endif
        enddo
    enddo

end subroutine

subroutine VVIntegrate(Pos, Vel, Accel, M, L, AA, rc, rlj, rw, sigma, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, rlj, rw, dt, sigma
    real(8), intent(in), dimension(0:NAtom-1) :: AA
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, AA, M, L, rc, rlj, rw, sigma, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine

subroutine Distance(Pos, L, Dim, NAtom, Dist)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L
    real(8), intent(out), dimension(0:NAtom-1, 0:NAtom-1)::Dist
    real(8), Dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12
    integer :: i, j
    Dist = 0.
    do i=0, NAtom-1
        Posi = Pos(i,:)
            !create temporary array to store Pos(i,:) for faster access in loop
        do j = 0, Natom - 1
            rij = Pos(j,:) - Posi
        	rij = rij - L * dnint(rij/L)
        	d2 = sum(rij * rij)
		Dist(i,j) = sqrt(d2)
	end do
  end do
end subroutine	
		
   